Option Strict Off
Option Explicit On 

Imports System
Imports System.IO
Imports System.Runtime.InteropServices
Imports System.Text



Module FL_Code

    Public ExtractMode As Short
    Public MainFrm As Form1
    Public FirstTimeForOpenDialog As Boolean = True

    Public Structure GLOBAL_VARS
        Public AppTitle As String
        Public DrawingPath As String
        Public FLCursors As CURSOR_FILES
        Public GlobalINI As String
        Public UserSettingsFolder As String
        Public UserINI As String
        Public LogFolder As String
        Public FastLookEXE As String
    End Structure

    Public Structure CURSOR_FILES
        Public PanMode As String
        Public PanGrab As String
    End Structure

    Public Structure USER_SETTINGS
        Public FormX As Integer
        Public FormY As Integer
        Public FormW As Integer
        Public FormH As Integer
        Public FormMaximized As Boolean

    End Structure
    Public Structure FIELD_INFO
        Public FieldName As String
        Public DisplayName As String
    End Structure

    Public FldInfo() As FIELD_INFO
    Public uVars As USER_SETTINGS
    Public gVars As GLOBAL_VARS

    Public DataPoint As Point
    Public NameWidth As Integer
    Public FieldNames() As String
    Public FieldValues() As String

    Public lblNames() As Label
    Public txtData() As TextBox
    Public InBlockSelectionMode As Boolean = False
    Public ScreenWidth, ScreenHeight As Integer
    Public IsDebug As Boolean

    Public Structure Preview_Object
        Public Left As Integer
        Public Top As Integer
        Public Width As Integer
        Public Height As Integer
        Public IsExpanded As Boolean
    End Structure

    Public Structure EXTENTS_STRUCT
        Public X1 As Double
        Public Y1 As Double
        Public X2 As Double
        Public Y2 As Double
    End Structure

    Public CurWindowExtents As EXTENTS_STRUCT
    Public FLWindowHeight, FLWindowWidth As Integer
    Public Channel, FastLookVersion As Integer
    Public FLEngineHndl As IntPtr
    Public FastLookDir As String
    Public FLObj As StartFastLook

    Public Sub Main()

        Try
            FastLookVersion = 15

            FLObj = New StartFastLook()
            AddHandler FLObj.FastLookEvent, AddressOf FL_Event_Handler

            gVars.AppTitle = "FastLook Simple Interface"
            gVars.GlobalINI = Path.GetDirectoryName(Application.ExecutablePath) & "\Simple.ini"
            Try
                gVars.UserSettingsFolder = Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData) & "\FastLookSimple\"
            Catch exs As Security.SecurityException
                MsgboxA("Failed to retrieve local settings path due to security: " & exs.ToString)
                HandleError(False, exs, "", "Main")
                End
            Catch ex As Exception
                MsgboxA("Failed to retrieve local settings path: " & ex.ToString)
                HandleError(False, ex, "", "Main")
                End
            End Try
            gVars.UserINI = "Simple.ini"
            IsDebug = False
            gVars.LogFolder = Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData) & "\FastLookSimple\logs\"
            MainFrm = New Form1

            Dim H, X, Y, W As Integer
            Application.EnableVisualStyles()

            Call GetSavedGlobalSettings()
            Call GetSavedUserSettings()
            MainFrm.Text = gVars.AppTitle
            MainFrm.WindowState = System.Windows.Forms.FormWindowState.Normal

            X = 0
            Y = 0
            H = MainFrm.pnlFastLook.Height
            W = MainFrm.pnlFastLook.Width
            FLObj.FastLookVersion = FastLookVersion
            Channel = FLObj.OpenFastLookWindow(MainFrm.pnlFastLook, X, Y, W, H)

            If Channel < 1 Then 'failed to open window
                MainFrm.Close()
                Exit Sub
            End If
            SetFLConfigWithHelp(Channel, fl_cfg.SUPPRESS_WINDOW_BORDER, 1, "")
            SetCursors(Channel)
            Call CheckMenusAndButtons()
            'SetFLConfigWithHelp(Channel, fl_cfg.USE_LAYOUT_TABS, 1, "")
            gVars.FastLookEXE = FLObj.FastLookEXE
            Call CreateRightClickMenu()
           
            Application.Run(MainFrm)
        Catch ex As Exception
            'MsgBox(ex.ToString)
            HandleError(True, ex, "", "Main")
        End Try


    End Sub

    Public Function WindowExtentsChanged() As Boolean
        Dim PrevExtents As EXTENTS_STRUCT = CurWindowExtents
        GetCurrentWindowExtents()
        If PrevExtents.Equals(CurWindowExtents) Then
            Return False
        Else
            Return True
        End If
    End Function

    Public Sub GetCurrentWindowExtents()
        Dim X1, Y1, X2, Y2 As Double
        Dim retVal As Integer
        retVal = GetWindowCoords(Channel, X1, Y1, X2, Y2)

        CurWindowExtents.X1 = X1
        CurWindowExtents.Y1 = Y1
        CurWindowExtents.X2 = X2
        CurWindowExtents.Y2 = Y2
    End Sub

    Public Function FromBaseUnitsToPixelsX(ByVal BaseUnitValue As Double) As Integer
        Dim BaseWidth, ThisDist, PixelsPerUnit As Double
        If CurWindowExtents.X2 > CurWindowExtents.X1 Then
            BaseWidth = CurWindowExtents.X2 - CurWindowExtents.X1
            PixelsPerUnit = FLWindowWidth / BaseWidth
            ThisDist = BaseUnitValue - CurWindowExtents.X1
            Return Int(PixelsPerUnit * ThisDist)
        Else
            BaseWidth = CurWindowExtents.X1 - CurWindowExtents.X2
            ThisDist = BaseUnitValue - CurWindowExtents.X2
            PixelsPerUnit = FLWindowWidth / BaseWidth
            Return Int(PixelsPerUnit * ThisDist)
        End If
    End Function

    Public Function FromBaseUnitsToPixelsY(ByVal BaseUnitValue As Double) As Integer
        Dim BaseHeight, ThisDist, PixelsPerUnit As Double
        If CurWindowExtents.Y2 > CurWindowExtents.Y1 Then
            BaseHeight = CurWindowExtents.Y2 - CurWindowExtents.Y1
            ThisDist = BaseUnitValue - CurWindowExtents.Y1
            PixelsPerUnit = FLWindowHeight / BaseHeight
            Return Int(PixelsPerUnit * ThisDist)
        Else
            BaseHeight = CurWindowExtents.Y1 - CurWindowExtents.Y2
            ThisDist = CurWindowExtents.Y1 - BaseUnitValue
            PixelsPerUnit = FLWindowHeight / BaseHeight
            Return Int(PixelsPerUnit * ThisDist)
        End If
    End Function

    Public Function FLRectToPixelRect(ByVal X1 As Double, ByVal Y1 As Double, ByVal X2 As Double, ByVal Y2 As Double) As Rectangle
        Dim BaseHeight, BaseWidth, PixelsPerUnit As Double
        Dim thisRect As New Rectangle(0, 0, 0, 0)
        If Y2 > Y1 Then
            BaseHeight = CurWindowExtents.Y2 - CurWindowExtents.Y1
            PixelsPerUnit = FLWindowHeight / BaseHeight
            thisRect.Y = CInt((Y1 - CurWindowExtents.Y1) * PixelsPerUnit)
            thisRect.Height = Int((Y2 - Y1) * PixelsPerUnit)

        Else
            BaseHeight = CurWindowExtents.Y1 - CurWindowExtents.Y2
            PixelsPerUnit = FLWindowHeight / BaseHeight
            thisRect.Y = CInt((CurWindowExtents.Y1 - Y1) * PixelsPerUnit)
            thisRect.Height = Int((Y1 - Y2) * PixelsPerUnit)
        End If
        If X2 > X1 Then
            BaseWidth = CurWindowExtents.X2 - CurWindowExtents.X1
            PixelsPerUnit = FLWindowWidth / BaseWidth
            thisRect.X = CInt((X1 - CurWindowExtents.X1) * PixelsPerUnit)
            thisRect.Width = Int((X2 - X1) * PixelsPerUnit)

        Else
            BaseWidth = CurWindowExtents.X1 - CurWindowExtents.X2
            PixelsPerUnit = FLWindowWidth / BaseWidth
            thisRect.X = CInt((CurWindowExtents.X1 - X1) * PixelsPerUnit)
            thisRect.Width = Int((X1 - X2) * PixelsPerUnit)

        End If
        Return thisRect
    End Function

    Public Sub ViewFile(ByVal FileName As String)
        If File.Exists(FileName) Then
            DrawDwgFile(Channel, FileName)
        Else
            MsgboxA("Cannot find: """ & FileName & """")
        End If
    End Sub


    Public Sub LaunchFastLook()
        Dim Answer As New StringBuilder(256)
        Dim ThisFile As String
        GetDwgName(Channel, Answer)
        ThisFile = Answer.ToString
        ThisFile = """" & ThisFile & """"
        Process.Start(FLObj.FastLookEXE, ThisFile)
    End Sub

    Private Sub CreateRightClickMenu()
        Dim retVal As Integer

        retVal = MouseClickAction(Channel, 3, 2, 23, 0, "Pan")
        retVal = MouseClickAction(Channel, 3, 2, 437, 0, "Layouts...")
        retVal = MouseClickAction(Channel, 3, 2, 10, 0, "Zoom Extents")
        retVal = MouseClickAction(Channel, 3, 2, 7, 0, "Print")
    End Sub


    Private Sub GetSavedUserSettings()
        'MsgBox("in GetSavedUserSettings")
        Dim fs As FileStream = Nothing
        Dim sr As StreamReader = Nothing
        If File.Exists(gVars.UserSettingsFolder & gVars.UserINI) Then
            Try
                fs = New FileStream(gVars.UserSettingsFolder & gVars.UserINI, FileMode.Open)
                fs.Seek(0, SeekOrigin.Begin)
                sr = New StreamReader(fs)

                Dim ThisLine As String
                Dim args() As String


                Do Until sr.EndOfStream
                    ThisLine = sr.ReadLine()
                    If ThisLine <> "" Then
                        args = ThisLine.Split("=")
                        If args.Length = 2 Then
                            Select Case LCase(args(0))
                                Case "formx"
                                    uVars.FormX = Int(args(1))
                                Case "formy"
                                    uVars.FormY = Int(args(1))
                                Case "formw"
                                    uVars.FormW = Int(args(1))
                                Case "formh"
                                    uVars.FormH = Int(args(1))
                                Case "formmax"
                                    If Int(args(1)) Then
                                        uVars.FormMaximized = True
                                    End If

                            End Select
                        End If
                    End If

                Loop

            Catch ex As Exception
                HandleError(True, ex, "", "GetSavedUserSettings")
            Finally
                If Not sr Is Nothing Then
                    sr.Close()
                End If
                If Not fs Is Nothing Then
                    fs.Close()
                End If
            End Try

        Else 'set defaults

        End If


    End Sub

    Public Sub SaveUserSettings()
        Dim fs As FileStream = Nothing
        Dim sw As StreamWriter = Nothing
        Try

            If Directory.Exists(gVars.UserSettingsFolder) = False Then
                Directory.CreateDirectory(gVars.UserSettingsFolder)
            End If
            fs = New FileStream(gVars.UserSettingsFolder & gVars.UserINI, FileMode.OpenOrCreate)
            fs.SetLength(0)

            sw = New StreamWriter(fs)
            sw.WriteLine("[Settings]")
            sw.WriteLine("FormX=" & MainFrm.Left.ToString)
            sw.WriteLine("FormY=" & MainFrm.Top.ToString)
            sw.WriteLine("FormW=" & MainFrm.Width.ToString)
            sw.WriteLine("FormH=" & MainFrm.Height.ToString)
            If MainFrm.WindowState = FormWindowState.Maximized Then
                sw.WriteLine("FormMax=1")
            Else
                sw.WriteLine("FormMax=0")
            End If



        Catch ex As Exception
            HandleError(True, ex, "", "SaveUserSettings")
        Finally
            If Not sw Is Nothing Then
                sw.Close()
            End If
            If Not fs Is Nothing Then
                fs.Close()
            End If
        End Try

    End Sub


    Private Sub GetSavedGlobalSettings()

        Dim fs As FileStream = Nothing
        Dim sr As StreamReader = Nothing
        Try
            If File.Exists(gVars.GlobalINI) = False Then
                'MsgboxA("The global INI does not exist!")
                Exit Sub
            End If
            fs = New FileStream(gVars.GlobalINI, FileMode.Open)
            sr = New StreamReader(fs)
            Dim ThisLine As String
            Dim args() As String

            fs.Seek(0, SeekOrigin.Begin)
            Do Until sr.EndOfStream
                ThisLine = sr.ReadLine()
                If ThisLine <> "" Then
                    args = ThisLine.Split("=")
                    If args.Length = 2 Then
                        Select Case LCase(args(0))
                            Case "drawingpath"
                                gVars.DrawingPath = args(1)
                            Case "fastlookversion"
                                FastLookVersion = Val(args(1))

                        End Select
                    End If
                End If

            Loop

            If gVars.DrawingPath = "" Then
                MsgboxA("The path to your drawings has not been set!")
                End
            End If
            If Directory.Exists(gVars.DrawingPath) = False Then
                MsgboxA("The path to your drawings does not exist!")
                End
            End If
            If gVars.DrawingPath.EndsWith("\") = False Then
                gVars.DrawingPath = gVars.DrawingPath & "\"
            End If

        Catch ex As Exception
            HandleError(True, ex, "", "GetSavedGlobalSettings")
        Finally
            If Not sr Is Nothing Then
                sr.Close()
            End If
            If Not fs Is Nothing Then
                fs.Close()
            End If


        End Try
    End Sub

    Public Sub FL_Event_Handler(ByVal ChannelNo As Integer, ByVal EventNumber As Integer, ByVal EventData As IntPtr) 'As Boolean
        'Debug.WriteLine("EventHandler: Channel = " & ChannelNo & ", EventNo = " & EventNumber)
        Dim retVal As Integer
        Dim LinkHandle As Integer, LinkType As Integer
        Dim Answer As StringBuilder

        Select Case EventNumber
            Case 2 'user doubleclicked in FastLook Window
                If EventData.Equals(IntPtr.Zero) Then
                    Dim extData As Integer
                    extData = Marshal.ReadInt32(EventData)
                    Marshal.FreeHGlobal(EventData)
                End If
            Case 3 'user invoked a database link created by the MakeLink API
                Dim DataStr As String, CaptionStr As String
                LinkHandle = GetLinkHandle(Channel)
                Answer = New StringBuilder(256)
                retVal = GetLinkData(Channel, LinkHandle, 6, Answer)
                DataStr = Answer.ToString
                Answer.Length = 0
                retVal = GetLinkData(Channel, LinkHandle, 5, Answer)
                CaptionStr = Answer.ToString
                MsgBox("Handle: " & LinkHandle.ToString & vbCrLf & "String: " & DataStr & vbCrLf & "Caption: " & CaptionStr)
            Case 4 'Beginning to redraw window
            Case 5 'viewing process has been completed, or redraw completed
                Call CheckMenusAndButtons()
            Case 6 'Redline attribute data is ready
            Case 7 'Polygon selected for area query
            Case 8 'User followed a FastLook link

            Case 9 'User closed the measure window

            Case 10 'Data has been found and is ready to be extracted
                Call ExtractData()

            Case 11  'application defined link selected for edit

                LinkHandle = GetLinkHandle(Channel)
                Answer = New StringBuilder(256)
                LinkType = GetLinkData(Channel, LinkHandle, 0, Answer)
                If LinkType = 3 Then 'application defined link - created by MakeLink API
                    Dim ThisFrm As New MakeLinkForm
                    ThisFrm.Tag = "EditLink"
                    ThisFrm.ShowDialog()
                End If
            Case 12 'Application defined link created
            Case 13 'File has been "dragged and dropped" on the FastLook window
                Dim i As Integer
                Dim FileCount As Integer
                Answer = New StringBuilder(256)
                FileCount = GetFirstDroppedFile(ChannelNo, Answer)
                If FileCount > 0 Then
                    If FileCount = 1 Then
                        Call ViewFile(Answer.ToString)
                    Else
                        Call ViewFile(Answer.ToString)
                        'might wish to walk through all dropped files
                        Dim FileList(FileCount) As String
                        For i = 1 To FileCount
                            FileList(i) = Answer.ToString
                            Answer.Length = 0
                            retVal = GetNextDroppedFile(ChannelNo, Answer)
                        Next
                    End If
                End If
            Case 14 'FastLook needs a new Link window
            Case 15 'OverLook window was closed
            Case 16 'OverLook window - starting to draw
            Case 17 'OverLook window - view completed
            Case 18 'Print server has finished ands shut down
            Case 19 'Export server has finished and shut down
            Case 20 'User canceled the print dialog
            Case 21 'Link file created
                Call CheckMenusAndButtons()
            Case 22 'Redline file is created
            Case 23 'Measure is ready to be retrieved
            Case 24 'User single-clicked in FastLook window
                If EventData.Equals(IntPtr.Zero) Then
                    Dim extData As Integer
                    extData = Marshal.ReadInt32(EventData)
                    Marshal.FreeHGlobal(EventData)
                End If
               
            Case 25 'User right-clicked while in MouseClickAction mode
                If EventData.Equals(IntPtr.Zero) Then
                    Dim extData As Integer
                    extData = Marshal.ReadInt32(EventData)
                    Marshal.FreeHGlobal(EventData)
                End If
            Case 26 'Entity pick mode has been canceled
                InBlockSelectionMode = False
                Call CheckMenusAndButtons()

            Case 27 'Redline file status changed
            Case 28 'Redline/Links selections set changed
            Case 29 'Redline or Link mode exited
                Call CheckMenusAndButtons()
            Case 30 'FastLook has just viewed a new image file
                Answer = New StringBuilder(256)
                retVal = GetDwgName(Channel, Answer)

                MainFrm.Text = gVars.AppTitle & ": Viewing: " & UCase(Path.GetFileName(Answer.ToString))

            Case 31 'User clicked on custom context menu item
            Case 32 'Print queue has been processed
            Case 33 'Curser moved over a link

            Case 34 'Curser moved off a link

            Case 35 'Link file status changed
            Case 36 'Export queue was completed
            Case 37 'User drew a "ghost" rectangle over the drawing window, after SendFLCommand #726 was called to trap this action - rectangle coordinates are ready to be retrieved
            Case 38  'Redline entity added or deleted
            Case 39 'Temp file unzipped and ready to view
                Answer = New StringBuilder(256)
                retVal = GetLinkString(ChannelNo, Answer)

                If Answer.ToString <> "" Then
                    ViewFile(Answer.ToString)
                End If
            Case 42 'Layer configuration changed
            Case 43 'User pressed a Key
                If EventData.Equals(IntPtr.Zero) Then
                    Exit Sub
                End If
                Dim KeyNum, lParam, extData As Integer
                KeyNum = Marshal.ReadInt32(EventData)
                lParam = Marshal.ReadInt32(EventData, 4)
                extData = Marshal.ReadInt32(EventData, 8) 'bits( Shift=1,CTRL=2, ALT=4)
                Marshal.FreeHGlobal(EventData)

                Select Case KeyNum
                    Case Keys.Enter
                        
                    Case Keys.Left, Keys.Right, Keys.Up, Keys.Down

                    Case 48 To 57, 65 To 90 '0-9, a-z
                        
                End Select
            Case 44 'link selection changed
            Case 45 'Print selection start
            Case 46 'Print selection end
            Case 47 'bypass link/redline selection mode menu
            Case 48 'bypass selected link/redline menu
            Case 49 'in link selection mode and nothing was selected
            Case 50 'in redline selection mode and nothing was selected
            Case 51 'Link was resized or moved
            Case 52 'Redline Symbol File changed
            Case 53 'Link Symbol File changed
            Case 54 'Curser moved over a link while in selection mode
            Case 55 'Curser moved off a link while in selection mode
            Case 56 'Layer status changed

        End Select
        'Return True
    End Sub

    Private Sub SetCursors(ByVal thisChannel As Integer)
        Dim retVal As Integer
        Dim strNameSpace As String = MainFrm.GetType().Namespace
        Dim resourseName As String
        Dim Str As Stream
        Dim fs As FileStream
        Dim br As BinaryWriter
        Dim tempFile As String
        Dim NeedToCreatePanMode As Boolean = False
        Dim NeedToCreatePanGrab As Boolean = False
        Try
            If gVars.FLCursors.PanMode Is Nothing Then
                NeedToCreatePanMode = True
            ElseIf File.Exists(gVars.FLCursors.PanMode) = False Then
                NeedToCreatePanMode = True
            End If
            If NeedToCreatePanMode Then
                resourseName = "HandOpen.cur"
                Str = System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(strNameSpace + "." + resourseName)
                If Not Str Is Nothing Then
                    If Str.CanRead Then
                        'bring stream into a byte array
                        tempFile = GetTempFile("", "FL_")
                        fs = New FileStream(tempFile, FileMode.CreateNew)
                        br = New BinaryWriter(fs)
                        Dim bStr(CInt(Str.Length)) As Byte
                        Str.Read(bStr, 0, CInt(Str.Length))
                        br.Write(bStr)
                        gVars.FLCursors.PanMode = tempFile
                        br.Close()
                        fs.Close()

                    End If
                End If
            End If
            If gVars.FLCursors.PanGrab Is Nothing Then
                NeedToCreatePanGrab = True
            ElseIf File.Exists(gVars.FLCursors.PanGrab) = False Then
                NeedToCreatePanGrab = True
            End If
            If NeedToCreatePanGrab Then
                resourseName = "HandClosed.cur"
                Str = System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(strNameSpace + "." + resourseName)
                If Not Str Is Nothing Then
                    If Str.CanRead Then
                        'bring stream into a byte array
                        tempFile = GetTempFile("", "FL_")
                        fs = New FileStream(tempFile, FileMode.CreateNew)
                        br = New BinaryWriter(fs)
                        Dim bStr(CInt(Str.Length)) As Byte
                        Str.Read(bStr, 0, CInt(Str.Length))
                        br.Write(bStr)
                        gVars.FLCursors.PanGrab = tempFile
                        br.Close()
                        fs.Close()
                    Else
                        DebugOutput("*** SetCursors: could not read stream for " & resourseName)
                    End If
                Else
                    DebugOutput("*** SetCursors: got nothing from the resource stream for " & resourseName)
                    Dim args() As String = System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceNames
                    For i As Integer = 0 To args.Length - 1
                        DebugOutput("*** SetCursors: resource " & i.ToString & " = " & args(i))
                    Next
                End If
            End If
            retVal = FLSetCursor(thisChannel, 1, gVars.FLCursors.PanMode)
            retVal = FLSetCursor(thisChannel, 2, gVars.FLCursors.PanGrab)


        Catch ex As Exception
            HandleError(True, ex, , "SetCursors")
        End Try
    End Sub

    Public Function GetTempFile(Optional ByVal FilePath As String = "", Optional ByVal FileNamePrefix As String = "") As String
        Dim TempPath As String, FileName As String
        Dim i As Integer
        Try

            If FilePath = "" Then
                TempPath = Path.GetTempPath
            Else
                TempPath = FilePath
            End If
            If FileNamePrefix = "" Then
                FileNamePrefix = "FL-"
            End If


            For i = 0 To 100
                FileName = TempPath & FileNamePrefix & System.Environment.TickCount & i.ToString & ".tmp"
                If File.Exists(FileName) = False Then
                    Return FileName
                End If
            Next i

            Return ""
        Catch ex As Exception
            HandleError(True, ex, "", "GetTempFile")
            Return ""
        End Try
    End Function

    

    Sub CheckMenusAndButtons()

        Dim LinkingIsOn, LinkFileExists, IsViewingFile, InRedlineSelect, InLinkSelect As Integer
        Dim RedliningIsOn, PrevLinkExists As Integer
        Dim IsACAD, ZoomPrevOK, LayersOK As Integer
        Dim Answer As New StringBuilder(256)
        Dim NumLayouts, retVal As Integer
        Try

            RedliningIsOn = GetFLStatusWithHelp(Channel, fl_status.REDLINE_ON)
            LinkingIsOn = GetFLStatusWithHelp(Channel, fl_status.LINKING_ON)
            LinkFileExists = GetFLStatusWithHelp(Channel, fl_status.LINK_FILE_EXISTS)
            IsACAD = GetFLStatusWithHelp(Channel, fl_status.IS_ACAD)
            ZoomPrevOK = GetFLStatusWithHelp(Channel, fl_status.CAN_ZOOM_PREVIOUS)
            LayersOK = GetFLStatusWithHelp(Channel, fl_status.SUPPORTS_LAYERS)
            NumLayouts = GetFLStatusWithHelp(Channel, fl_status.NUM_LAYOUTS)
            PrevLinkExists = GetFLStatusWithHelp(Channel, fl_status.CAN_RETRACE_LINK)

            retVal = GetFLStatusWithHelp(Channel, fl_status.SELECTION_MODE)
            Select Case retVal
                Case 1
                    InRedlineSelect = 1
                    InLinkSelect = 0
                Case 2
                    InRedlineSelect = 0
                    InLinkSelect = 1
                Case Else
                    InRedlineSelect = 0
                    InLinkSelect = 0
            End Select
            retVal = GetDwgName(Channel, Answer)
            If Answer.ToString = "" Then 'no file is being viewed
                IsViewingFile = 0
            Else
                IsViewingFile = 1
            End If

            Dim ScrollBarsOn, LayoutTabsOn, GasGaugeOn As Integer
            ScrollBarsOn = GetFLStatusWithHelp(Channel, fl_status.SCROLL_BARS_ON)
            LayoutTabsOn = GetFLStatusWithHelp(Channel, fl_status.USE_LAYOUT_TABS)
            GasGaugeOn = GetFLStatusWithHelp(Channel, fl_status.GAS_GAUGE_ON)

            FLWindowHeight = MainFrm.pnlFastLook.Height
            FLWindowWidth = MainFrm.pnlFastLook.Width
            If ScrollBarsOn Then
                FLWindowHeight = MainFrm.pnlFastLook.Height - 16
                FLWindowWidth = MainFrm.pnlFastLook.Width - 16
            End If
            If GasGaugeOn Then
                FLWindowWidth = FLWindowWidth - 8
            End If
            If LayoutTabsOn Then
                FLWindowHeight = MainFrm.pnlFastLook.Height - 16
            End If


            MainFrm.btnRedOn.Checked = -1 * RedliningIsOn

            MainFrm.btnPrint.Enabled = -1 * IsViewingFile
            If NumLayouts > 1 Then
                MainFrm.btnLayouts.Enabled = True
            Else
                MainFrm.btnLayouts.Enabled = False
            End If

            MainFrm.btnPan.Enabled = -1 * IsViewingFile
            MainFrm.btnZoomOut.Enabled = -1 * IsViewingFile
            MainFrm.btnZoomIn.Enabled = -1 * IsViewingFile
            MainFrm.btnZoomPrev.Enabled = -1 * ZoomPrevOK
            MainFrm.btnZoomExt.Enabled = -1 * IsViewingFile

            MainFrm.btnLinkingOn.Checked = -1 * LinkingIsOn
            MainFrm.btnPrevLink.Enabled = -1 * LinkingIsOn * PrevLinkExists
            MainFrm.btnLinkingBox.Enabled = -1 * LinkingIsOn * IsViewingFile
            MainFrm.btnSymbolLink.Enabled = -1 * LinkingIsOn * IsViewingFile
            MainFrm.btnSelectLink.Enabled = -1 * LinkingIsOn * IsViewingFile
            MainFrm.btnSelectLink.Checked = -1 * InLinkSelect
            MainFrm.mLinkingOn.Checked = -1 * LinkingIsOn
            MainFrm.mMakeLinkAPI.Enabled = -1 * LinkingIsOn * IsViewingFile
            MainFrm.mEditUserDefinedLink.Enabled = -1 * LinkingIsOn * IsViewingFile

            MainFrm.btnRedOn.Checked = -1 * RedliningIsOn
            MainFrm.mRedlineOn.Checked = -1 * RedliningIsOn
            MainFrm.btnSelectRedline.Enabled = -1 * RedliningIsOn * IsViewingFile
            MainFrm.btnSelectRedline.Checked = -1 * InRedlineSelect
            MainFrm.btnBox.Enabled = -1 * RedliningIsOn * IsViewingFile
            MainFrm.btnCircle.Enabled = -1 * RedliningIsOn * IsViewingFile
            MainFrm.btnLine.Enabled = -1 * RedliningIsOn * IsViewingFile
            MainFrm.btnNote.Enabled = -1 * RedliningIsOn * IsViewingFile
            MainFrm.btnRedPreferences.Enabled = -1 * RedliningIsOn * IsViewingFile

            MainFrm.btnBlockAttribute.Enabled = -1 * IsACAD * IsViewingFile
            MainFrm.btnListBlockAttribute.Enabled = -1 * IsACAD * IsViewingFile
            MainFrm.btnListTextEntities.Enabled = -1 * IsACAD * IsViewingFile

        Catch ex As Exception
            HandleError(True, ex, "", "CheckMenusAndButtons")
        End Try
    End Sub


    Sub ExtractData()
        Dim Tag As StringBuilder
        Dim Value As StringBuilder

        Dim Success As Integer
        Dim LinkComplete, AnotherLink, NumLinks, ACADversion As Short
        Dim SQLValue, SQLTag, Msg, TagNew, ValueNew As String
        Dim DatabaseType(10), DatabasePath(10), DatabaseName(10) As String
        Dim DataFieldName(10), DataType(10), DataFieldValue(10) As String

        Msg = ""
        Try
            If ExtractMode = 1 Then
                Tag = New StringBuilder(256)
                Value = New StringBuilder(256)
                Success = GetFirstEntityData(Channel, Tag, Value)
                SQLTag = Tag.ToString
                SQLValue = Value.ToString
                ' MsgBox "Tag is: " & SQLTag
NextSQLLink:
                Select Case SQLTag
                    Case "App_Name"
                        NumLinks = NumLinks + 1
                        If InStr(SQLValue, "R13") Then
                            ACADversion = 13
                        Else
                            ACADversion = 12
                        End If
                    Case "DBType"
                        DatabaseType(NumLinks) = SQLValue
                    Case "Symbolic_DB_Path"
                        'use this to resolve the path if FastLook could not
                    Case "Actual_DB_Path"
                        DatabasePath(NumLinks) = SQLValue
                    Case "Table_Name"
                        DatabaseName(NumLinks) = SQLValue
                    Case "Field_Name"
                        DataFieldName(NumLinks) = SQLValue
                    Case "Field_Value"
                        DataFieldValue(NumLinks) = SQLValue
                End Select

                While Success
                    Tag = New StringBuilder(256)
                    Value = New StringBuilder(256)
                    Success = GetNextEntityData(Channel, Tag, Value)
                    SQLTag = Tag.ToString
                    SQLValue = Value.ToString
                    If Success Then
                        Select Case SQLTag
                            Case "App_Name"
                                NumLinks = NumLinks + 1
                                If InStr(SQLValue, "R13") Then
                                    ACADversion = 13
                                Else
                                    ACADversion = 12
                                End If
                            Case "DB_Type"
                                DatabaseType(NumLinks) = SQLValue
                            Case "Symbolic_DB_Path"
                                'use this to resolve the path if FastLook could not
                            Case "Actual_DB_Path"
                                DatabasePath(NumLinks) = SQLValue
                            Case "Table_Name"
                                DatabaseName(NumLinks) = SQLValue
                            Case "Field_Name"
                                DataFieldName(NumLinks) = SQLValue
                            Case "Field_Value"

                                'If IsNothing(SQLValue) Then
                                If SQLValue.Length = 0 Then
                                    MsgBox("No data value was found!")

                                    Success = 0
                                Else
                                    DataFieldValue(NumLinks) = SQLValue
                                    Msg = "Database Name is:  " + DatabaseName(NumLinks)
                                    Msg = Msg & Chr(10) & Chr(13) & "Database Type is:   " & DatabaseType(NumLinks)
                                    Msg = Msg & Chr(10) & Chr(13) & "Database Path is:   " & DatabasePath(NumLinks)
                                    Msg = Msg & Chr(10) & Chr(13) & "Data Field is:      " & DataFieldName(NumLinks)
                                    Msg = Msg & Chr(10) & Chr(13) & "Field Value is:     " & DataFieldValue(NumLinks)
                                    Msg = Msg & Chr(10) & Chr(13) & "Data Type is:       " & DataType(NumLinks)
                                    Msg = Msg & Chr(10) & Chr(13) & "Link Number is:     " & NumLinks
                                    MsgBox(Msg)
                                    LinkComplete = 1
                                    Success = 0
                                End If
                        End Select
                    End If
                End While
                If LinkComplete Then
                    LinkComplete = 0
                    Success = 1
                    While Success
                        Tag = New StringBuilder(256)
                        Value = New StringBuilder(256)
                        Success = GetNextEntityData(Channel, Tag, Value)
                        SQLTag = Tag.ToString
                        SQLValue = Value.ToString
                        If SQLTag = "App_Name" Then
                            Success = 0
                            AnotherLink = 1
                        End If
                    End While
                    If AnotherLink Then
                        Success = 1
                        AnotherLink = 0
                        GoTo NextSQLLink
                    End If
                End If
            End If

            If ExtractMode = 2 Then
                Tag = New StringBuilder(256)
                Value = New StringBuilder(256)
                Success = GetFirstEntityData(Channel, Tag, Value)
                If Success Then
                    TagNew = Tag.ToString
                    ValueNew = Value.ToString

                    Msg = TagNew & " = " & ValueNew
                End If
                While Success
                    Tag = New StringBuilder(256)
                    Value = New StringBuilder(256)
                    Success = GetNextEntityData(Channel, Tag, Value)
                    If Success Then
                        TagNew = Tag.ToString
                        ValueNew = Value.ToString

                        Msg = Msg & Chr(10) & Chr(13) & TagNew & " = " & ValueNew
                    End If
                End While
                MsgBox(Msg)
            End If

        Catch ex As Exception
            MsgboxA(ex.ToString)

        End Try


    End Sub '



    Public Function ConvertString(ByVal OldString As String) As String
        If OldString <> "" Then
            Return Left(OldString, InStr(OldString, Chr(0)) - 1)
        Else
            Return ""
        End If

    End Function

    Public Sub HandleError(ByVal DisplayMsg As Boolean, ByVal exObj As Exception, Optional ByVal ErrorStr As String = "", Optional ByVal Procedure As String = "")
        Dim LogFile, LogStr, MsgStr As String
        MsgStr = ""
        If DisplayMsg = True Then
            If IsDebug Then
                If Not exObj Is Nothing Then
                    MsgStr = exObj.ToString
                Else
                    MsgStr = ErrorStr
                End If
            Else
                If ErrorStr <> "" Then
                    MsgStr = ErrorStr
                ElseIf Not exObj Is Nothing Then
                    MsgStr = exObj.Message
                End If
            End If
            MsgboxA(MsgStr)

        End If
        Try

            If gVars.LogFolder = "" Then
                Exit Sub
            End If
            If Directory.Exists(gVars.LogFolder) = False Then
                Directory.CreateDirectory(gVars.LogFolder)
            End If
            LogFile = gVars.LogFolder & "FLSimple_Error.log"
            Dim fs As FileStream
            fs = New FileStream(LogFile, FileMode.OpenOrCreate)
            If fs.Length > 200000 Then
                Try
                    fs.Close()
                    File.Delete(LogFile)
                    fs = New FileStream(LogFile, FileMode.OpenOrCreate)
                Catch ex As Exception
                End Try
            End If

            LogStr = Now.ToShortDateString & " " & Now.ToShortTimeString
            If Procedure <> "" Then
                LogStr = LogStr & " Procedure: " & Procedure
            End If

            If ErrorStr <> "" Then
                MsgStr = ErrorStr
                LogStr = LogStr & " Error: " & ErrorStr
            ElseIf Not exObj Is Nothing Then
                LogStr = LogStr & " Error: " & exObj.Message
                MsgStr = exObj.Message
            End If
            If Not exObj Is Nothing Then
                LogStr = LogStr & vbCrLf & " Debug: " & exObj.ToString
            End If
            LogStr = LogStr & vbCrLf



            Dim sw As StreamWriter = New StreamWriter(fs)
            fs.Seek(0, SeekOrigin.End)
            sw.WriteLine(LogStr)
            sw.Close()
            fs.Close()

            If IsDebug Then
                If ErrorStr <> "" Then
                    MsgStr = "Error: " & ErrorStr
                ElseIf Not exObj Is Nothing Then
                    MsgStr = "Error: " & exObj.Message
                End If
                Call DebugOutput(MsgStr)
            End If
        Catch ex As Security.SecurityException
            MsgboxA("No permission to write to log file!")
        Catch ex As Exception
            MsgboxA(ex.ToString)

        End Try
    End Sub

    Sub DebugOutput(ByVal OutputStr As String)
        Try
            If IsDebug Then
                Dim OutputFile As String ', FileNo As Long
                If Directory.Exists(gVars.LogFolder) = False Then
                    Directory.CreateDirectory(gVars.LogFolder)
                End If
                OutputFile = gVars.LogFolder & "FLSimple_Progress.log"

                If LCase(OutputStr) = "clear" Then
                    If File.Exists(OutputFile) Then
                        Kill(OutputFile)
                        'Exit Sub
                        OutputStr = Now '& Chr(13) & Chr(10)
                    End If
                Else
                    OutputStr = OutputStr '& Chr(13) & Chr(10)
                End If

                Dim fs As New FileStream(OutputFile, FileMode.OpenOrCreate)

                Dim sw As StreamWriter = New StreamWriter(fs)
                fs.Seek(0, SeekOrigin.End)
                sw.WriteLine(OutputStr)
                sw.Close()
                fs.Close()

            End If
            Debug.WriteLine(OutputStr)

        Catch ex As Security.SecurityException
            MsgboxA("No permission to write debug file!")
            IsDebug = False
        Catch ex As Exception
            MsgboxA(ex.ToString)
        End Try

    End Sub

    Public Function GetOptimumPlacementPoint(ByVal MainWindowRect As Rectangle, ByVal ChildWindowRect As Rectangle, ByVal RectToAttachTo As Rectangle) As Point
        ' this assumes the desire to border the bottom of the placement rect and start at the left of it.
        Dim thisPoint As New Point(0, 0)
        Debug.WriteLine("MainWindowRect = " & MainWindowRect.ToString)
        Debug.WriteLine("ChildWindowRect = " & ChildWindowRect.ToString)
        Debug.WriteLine("RectToAttachTo = " & RectToAttachTo.ToString)
        If MainWindowRect.Width - RectToAttachTo.Left > ChildWindowRect.Width Then 'can use the left
            thisPoint.X = RectToAttachTo.Left
        Else
            thisPoint.X = RectToAttachTo.Right
        End If
        If MainWindowRect.Height - RectToAttachTo.Bottom > ChildWindowRect.Height Then
            thisPoint.Y = RectToAttachTo.Bottom
        Else
            thisPoint.Y = RectToAttachTo.Top
        End If
        Return thisPoint
    End Function

    Public Sub MsgboxA(ByVal Msg As String)
        MsgBox(Msg, , gVars.AppTitle)
    End Sub
End Module