
Imports System.io
Imports System.Runtime.InteropServices
Imports Microsoft.Win32
Imports System.Text


Class StartFastLook
    Private FL_EventQ As New Queue

    Private Class FL_Event_Item
        Private mChannel, mEventNo As Integer
        Private mDataPtr As IntPtr = Nothing

        Public ReadOnly Property Channel() As Integer
            Get
                Return mChannel
            End Get
        End Property
        Public ReadOnly Property EventNo() As Integer
            Get
                Return mEventNo
            End Get
        End Property
        Public ReadOnly Property DataPtr() As IntPtr
            Get
                Return mDataPtr
            End Get
        End Property

        Public Sub New(ByVal ChannelNum As Integer, ByVal EventNum As Integer, ByVal Data As Integer)
            mChannel = ChannelNum
            mEventNo = EventNum
            Select Case EventNum
                Case 24, 25
                    If Data <> 0 Then
                        mDataPtr = Marshal.AllocHGlobal(4)
                        Marshal.WriteInt32(mDataPtr, 0, Marshal.ReadInt32(New IntPtr(Data)))
                    End If
                Case 43
                    If Data <> 0 Then
                        mDataPtr = Marshal.AllocHGlobal(8)
                        Debug.WriteLine("FL_Event_Item - keycode: " & Marshal.ReadInt32(New IntPtr(Data)).ToString & ", Lparam: " & Marshal.ReadInt32(New IntPtr(Data + 4)).ToString)
                        Marshal.WriteInt32(mDataPtr, 0, Marshal.ReadInt32(New IntPtr(Data)))
                        Marshal.WriteInt32(mDataPtr, 4, Marshal.ReadInt32(New IntPtr(Data + 4)))
                        Marshal.WriteInt32(mDataPtr, 8, Marshal.ReadInt32(New IntPtr(Data + 8)))
                    End If
            End Select
        End Sub
    End Class

    Private CB_Delegate As EventCallBackEx
    Private mFastLookVersion As Integer
    Private mFastLookDir As String
    Private mFastLookEXE As String
    Private mEventTimer As Timer

    Public Event FastLookEvent(ByVal ChannelNo As Integer, ByVal EventNumber As Integer, ByVal EventData As IntPtr)

    Public Sub New()
        mEventTimer = New Timer
        mEventTimer.Enabled = False
        mEventTimer.Interval = 50
        AddHandler mEventTimer.Tick, AddressOf mEventTimer_Tick
    End Sub

    Public ReadOnly Property FastLookEXE() As String
        Get
            Select Case mFastLookVersion
                Case 13
                    Return mFastLookDir & "\fl2009.exe"
                Case Else
                    Return mFastLookDir & "\fl" & mFastLookVersion.ToString & ".exe"
            End Select

        End Get
    End Property

    Public ReadOnly Property FastLookDir() As String
        Get
            Return mFastLookDir
        End Get
    End Property

    Public Property FastLookVersion() As Integer
        Get
            Return mFastLookVersion
        End Get
        Set(ByVal value As Integer)
            mFastLookVersion = value
        End Set
    End Property

    Public Function OpenFastLookWindow(ByVal ParentObj As Object, ByVal X As Integer, ByVal Y As Integer, ByVal W As Integer, ByVal H As Integer) As Integer
        Dim retVal, ThisChannel, i As Integer
        Dim WinHandle As IntPtr
        Dim EngineLoaded As Boolean
        If mFastLookVersion < 14 Then
            mFastLookVersion = 14
        End If

        Try
            WinHandle = ParentObj.Handle '.ToInt32
        Catch ex As Exception
            MsgboxA("The object you are trying to attach FastLook to does not have a window handle property ('hWind')!")
            Return 0
        End Try
        Try
            EngineLoaded = FindAndLoadFastLookEngine()
            If EngineLoaded = False Then
                Return 0
            End If

            ThisChannel = 1
            Dim retWind As IntPtr
            retWind = OpenDwgWindow(WinHandle, ThisChannel, X, Y, W, H)
            If retWind.ToInt64 = 2 Then 'channel in use
                For i = 1 To 20
                    ThisChannel = ThisChannel + 1
                    retWind = OpenDwgWindow(WinHandle, ThisChannel, X, Y, W, H)
                    If retWind.ToInt64 > 32 Or retWind.ToInt64 < 0 Then 'success (can be negative in 64bit)
                        Exit For
                    End If
                Next i
            End If
            If retWind.ToInt64 > 0 And retWind.ToInt64 < 32 Then 'must have errored (note: can be negative in 64bit)
                If retWind.ToInt64 = 10 Then
                    MsgboxA("The FastLook user count has been exceeded!")
                Else
                    MsgboxA("Could not open a FastLook window, error code: " & retWind.ToString)
                End If
                Return 0

            End If
            If CB_Delegate Is Nothing Then
                CB_Delegate = New EventCallBackEx(AddressOf FL_EventCallBackEx)
            End If
            retVal = SetEventCallBackEx(ThisChannel, CB_Delegate)
            Return ThisChannel
        Catch ex As Exception
            HandleError(True, ex, "", "OpenFastLookWindow")
            Return 0
        End Try
    End Function

    Private Function FindAndLoadFastLookEngine() As Boolean
        Dim FLEnginePath As String
        Dim i As Integer

#If IS_64_BIT = True Then
        Const FLEng_Dll As String = "flengine.dll"
#Else
    Const FLEng_Dll As String = "fleng32.dll"

#End If
        Try


            mFastLookDir = GetFLPath(FastLookVersion)
            If mFastLookDir = "" Then
                For i = mFastLookVersion - 1 To mFastLookVersion - 3 Step -1
                    mFastLookDir = GetFLPath(i)
                    If mFastLookDir <> "" Then
                        mFastLookVersion = i
                        Exit For
                    End If
                Next i
            End If
            If mFastLookDir = "" Then
                MsgBox("Could not find FastLook!")
                End
            End If

            FLEnginePath = mFastLookDir & FLEng_Dll
            FLEngineHndl = LoadLibrary(FLEnginePath)
            If FLEngineHndl = IntPtr.Zero Then
                MsgBox("Failed to load the FastLook Engine!")
                Return False
            End If
            Return True
        Catch ex As Exception
            HandleError(True, ex, "", "FindAndLoadFastLookEngine")
            Return False
        End Try
    End Function

    Private Function GetFLPath(ByVal WhichVersion As Integer) As String
        Dim thisPath As String, VerStr As String
        Dim FLUserIniFile As String
        Dim Answer As New StringBuilder(256)
        Dim retVal As Long
        Dim rk As RegistryKey
        Dim FLKey As RegistryKey
        Dim CurVer As String
        Dim FLKeyStr As String
        Dim AppPath As String
        Dim FLEnginePath As String

        thisPath = ""
        AppPath = Path.GetDirectoryName(Application.ExecutablePath)
#If IS_64_BIT Then
        FLEnginePath = AppPath & "\flengine.dll"
        FLKeyStr = "SOFTWARE\Kamel Software\FastLook\64Bit"
#Else
        FLEnginePath = AppPath & "\fleng32.dll"
        FLKeyStr = "SOFTWARE\Kamel Software\FastLook\32Bit"
#End If
        If File.Exists(FLEnginePath) Then
            DebugOutput("in GetFLPath: found engine in exe path")
            Return AppPath & "\"
        End If

        Select Case WhichVersion
            Case Is > 14
                rk = Registry.CurrentUser

                FLKey = rk.OpenSubKey(FLKeyStr)
                If Not FLKey Is Nothing Then
                    CurVer = FLKey.GetValue("CurVer")
                    If Val(CurVer) = WhichVersion Then
                        thisPath = FLKey.GetValue("Path")
                        If thisPath.EndsWith("\") = False Then
                            thisPath += "\"
                        End If
                        DebugOutput("in GetFLPath: found engine in SOFTWARE\Kamel Software\FastLook CurVer")
                    End If
                End If
                If thisPath = "" Then
                    FLKey = rk.OpenSubKey("SOFTWARE\Kamel Software\FastLook" & WhichVersion.ToString)
                    If Not FLKey Is Nothing Then
                        CurVer = FLKey.GetValue("Is64Bit")
#If IS_64_BIT Then
                        If CurVer = "1" Then
#Else
                        If CurVer = "0" Then
#End If
                            thisPath = FLKey.GetValue("Path")
                            If thisPath <> "" Then
                                thisPath = Path.GetDirectoryName(thisPath) & "\"
                                DebugOutput("in GetFLPath: found engine in SOFTWARE\Kamel Software\FastLook" & WhichVersion.ToString & ", fllaunch")
                            End If
                        End If
                    End If
                End If
                If thisPath = "" Then
                    FLUserIniFile = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData)
                    If FLUserIniFile <> "" Then
                        FLUserIniFile = FLUserIniFile & "\Kamel Software\FastLook V" & WhichVersion.ToString & "\FLUSER.INI"
                        If Dir(FLUserIniFile) <> "" Then
                            retVal = GetPrivateProfileString("FASTLOOK" & WhichVersion.ToString, "fllaunch", "", Answer, 256, FLUserIniFile)
                            If retVal > 1 Then
                                thisPath = Path.GetDirectoryName(Answer.ToString) & "\"
                                DebugOutput("in GetFLPath: found engine in FLUSER.INI, fllaunch")
                            End If
                            If Directory.Exists(thisPath) = False Then
                                thisPath = ""
                            End If
                        End If
                    End If
                End If

            Case 14
                rk = Registry.CurrentUser
                FLKey = rk.OpenSubKey("SOFTWARE\Kamel Software\FastLook14")
                If Not FLKey Is Nothing Then
                    thisPath = FLKey.GetValue("Path")
                    If thisPath <> "" And Right(thisPath, 1) <> "\" Then
                        thisPath = thisPath & "\"
                    End If
                End If
                If thisPath = "" Then
                    FLUserIniFile = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData)
                    If FLUserIniFile <> "" Then
                        FLUserIniFile = FLUserIniFile & "\Kamel Software\FastLook V14\FLUSER.INI"
                        If Dir(FLUserIniFile) <> "" Then
                            Answer = New StringBuilder(256)
                            retVal = GetPrivateProfileString("FASTLOOK14", "fllaunch", "", Answer, 256, FLUserIniFile)
                            If retVal > 1 Then
                                thisPath = Path.GetDirectoryName(Answer.ToString) & "\"
                            End If
                        End If
                    End If
                End If

            Case 13

                FLUserIniFile = Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData)
                If FLUserIniFile <> "" Then
                    FLUserIniFile = FLUserIniFile & "\Kamel Software\FastLook 2009\FLUSER.INI"
                    If File.Exists(FLUserIniFile) Then
                        Answer = New StringBuilder(256)
                        retVal = GetPrivateProfileString("FASTLOOK13", "fllaunch", "", Answer, 256, FLUserIniFile)
                        If retVal > 1 Then
                            thisPath = Path.GetDirectoryName(Answer.ToString) & "\"
                            If Directory.Exists(thisPath) Then
                                Return thisPath
                            End If
                        End If
                    End If
                End If

                FLUserIniFile = Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData)
                If FLUserIniFile <> "" Then
                    FLUserIniFile = FLUserIniFile & "\Kamel Software\FastLook 2009\FLUSER.INI"
                    If File.Exists(FLUserIniFile) Then
                        Answer = New StringBuilder(256)
                        retVal = GetPrivateProfileString("FASTLOOK13", "fllaunch", "", Answer, 256, FLUserIniFile)
                        If retVal > 1 Then
                            thisPath = Path.GetDirectoryName(Answer.ToString) & "\"
                            If Directory.Exists(thisPath) Then
                                Return thisPath
                            End If
                        End If
                    End If
                End If


                rk = Registry.LocalMachine
                FLKey = rk.OpenSubKey("SOFTWARE\Kamel Software\FastLook")
                If Not FLKey Is Nothing Then
                    VerStr = Val(FLKey.GetValue("CurVer"))
                    If Val(VerStr) = WhichVersion Then
                        thisPath = FLKey.GetValue("Path")
                        If thisPath <> "" And Right(thisPath, 1) <> "\" Then
                            thisPath = thisPath & "\"
                        End If
                        If Directory.Exists(thisPath) Then
                            Return thisPath
                        End If
                    End If
                End If


            Case Else
                MsgBox("Unsupported version of FastLook")
        End Select
        Return thisPath
    End Function

    Private Sub mEventTimer_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs)
        Call ProcessNextEvent()
    End Sub

    Private Sub ProcessNextEvent()
        mEventTimer.Enabled = False
        If FL_EventQ.Count > 0 Then
            Dim thisEvent As FL_Event_Item
            thisEvent = CType(FL_EventQ.Dequeue, FL_Event_Item)
            If Not thisEvent Is Nothing Then
                'Debug.WriteLine("Call: Channel = " & thisEvent.Channel & ", EventNo = " & thisEvent.EventNo & ", Data = " & thisEvent.DataPtr.ToString)
                RaiseEvent FastLookEvent(thisEvent.Channel, thisEvent.EventNo, thisEvent.DataPtr)
                mEventTimer.Enabled = True
            End If
        End If

    End Sub


    Private Sub FL_EventCallBackEx(ByVal ChannelNo As Integer, ByVal EventNumber As Integer, ByVal EventData As Integer)
        Try
            DebugOutput("Event: Channel = " & ChannelNo & ", EventNo = " & EventNumber & ", Data = " & EventData)
            Dim thisEvent As New FL_Event_Item(ChannelNo, EventNumber, EventData)

            Select Case EventNumber
                Case 4, 5, 30
                    If FL_EventQ.Contains(thisEvent) = False Then
                        FL_EventQ.Enqueue(thisEvent)
                    End If
                Case Else
                    FL_EventQ.Enqueue(thisEvent)
            End Select
            mEventTimer.Enabled = True

        Catch ex As Exception
            MsgBox(ex.ToString)

        End Try
    End Sub

End Class
